from rest_framework import serializers
from users.serializers import UserSerializer
from users.models import User


class GenericNotificationRelatedField(serializers.RelatedField):

    def to_representation(self, value):
        data = {"classname": value.__class__.__name__, "id": value.id}
        return data


class NotificationSerializer(serializers.Serializer):
    sender = UserSerializer(
        User, read_only=True
    )  # El que ejecuta el envio de la notificacion
    recipient = UserSerializer(User, read_only=True)  # El que la recibe
    unread = serializers.BooleanField(read_only=True)
    verb = serializers.CharField(read_only=True)
    description = serializers.CharField(read_only=True)
    timestamp = serializers.DateTimeField(read_only=True)
    action_object = GenericNotificationRelatedField(
        read_only=True
    )  # El elemento relacionado a la notificacion


# notify.send(actor, recipient, verb, action_object, target, level, description, public, timestamp, **kwargs)



from exponent_server_sdk import (
    DeviceNotRegisteredError,
    PushClient,
    PushMessage,
    PushTicketError,
)
from django_notifications_views.app_settings import api_settings as settings

if settings.get_user_setting('USE_EXPO_NOTIFICATIONS'):
    def send_push_message(token, message, data=None):
        """
        Send push notification to expo token associated with user.
        if token is not valid, delete it from db
        :param token: string
        :param message: string
        :param data:optional: dict
        :return: None
        """
        try:
            response = PushClient().publish(PushMessage(to=token, body=message, data=data))
        except Exception as e:
            print("Encountered push error: ", e)
        else:
            try:
                response.validate_response()

                print(response)
            except DeviceNotRegisteredError:
                from django_notifications_views.models import ExpoToken

                ExpoToken.objects.filter(token=token).delete()
            except PushTicketError as e:
                print("Encountered push error: ", e)


    def expo_push_notification_handler(recipient, verb, data, **kwargs):
        if not recipient.expo_tokens.exists():
            return

        tokens = recipient.expo_tokens.values_list("token", flat=True)
        for token in tokens:
            send_push_message(token, verb, data=data)



__title__ = 'Django-notifications-views'
__description__ = 'Django-notifications-views is an extension for Django-notifications-hq that provides a viewset for the notifications.'
__url__ = 'https://github.com/nachoborrelli/Django-notifications-views'
__version__ = '0.1.5'
__author__ = '@nachoborrelli https://github.com/nachoborrelli'
__author_email__ = 'nacho2911@hotmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2024 @nachoborrelli https://github.com/nachoborrelli'



from django.db import models
from notifications.signals import notify
from django_notifications_views.app_settings import api_settings as settings


if settings.get_user_setting('USE_EXPO_NOTIFICATIONS'):
    from django_notifications_views.expo_utils import expo_push_notification_handler
    class ExpoToken(models.Model):
        token = models.CharField(max_length=200)
        user = models.ForeignKey(
            "users.User", on_delete=models.CASCADE, related_name="expo_tokens"
        )
        created_at = models.DateTimeField(auto_now_add=True)

        def __str__(self):
            return f"{self.user} - {self.token}"


    notify.connect(
        expo_push_notification_handler, dispatch_uid="expo_push_notification_handler"
    )



from rest_framework.routers import DefaultRouter
from django_notifications_views.views import NotificationsViewSet
from django_notifications_views.app_settings import api_settings as settings


router = DefaultRouter()
router.register(r"user-notifications", NotificationsViewSet, basename="notifications")

if settings.get_user_setting("USE_EXPO_NOTIFICATIONS"):
    from django_notifications_views.views import ManageDeviceExpo
    router.register(r"expo-devices", ManageDeviceExpo, basename="expo")



from rest_framework.viewsets import GenericViewSet
from rest_framework.mixins import ListModelMixin, RetrieveModelMixin
from rest_framework import filters as rest_filters
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.exceptions import ValidationError

from django_filters import rest_framework as filters

from django.conf import settings

from django_notifications_views.serializers import NotificationSerializer

from django_notifications_views.app_settings import api_settings as settings

class NotificationsViewSet(GenericViewSet, ListModelMixin, RetrieveModelMixin):
    filter_backends = (
        filters.DjangoFilterBackend,
        rest_filters.SearchFilter,
        rest_filters.OrderingFilter,
    )

    filterset_fields = ("unread",)
    
    search_fields = ("description", "verb")

    page_size_query_param = "page_size"

    ordering_fields = ("update_at", "created_at")

    serializer_class = NotificationSerializer


    def get_queryset(self):
        return self.request.user.notifications.all()

    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        instance.unread = False
        instance.save()
        return super().retrieve(request, *args, **kwargs)

    @action(detail=False, methods=["post"], url_path="read-all")
    def mark_all_as_read(self, request):
        for notification in self.get_queryset():
            notification.unread = False
            notification.save()
        return Response(status=status.HTTP_204_NO_CONTENT)
    

if settings.get_user_setting('USE_EXPO_NOTIFICATIONS'):
    class ManageDeviceExpo(GenericViewSet):
        permission_classes = [IsAuthenticated]

        @action(detail=False, url_path="register-device", methods=["POST"])
        def register_device(self, request, *args, **kwargs):
            user = request.user

            token = request.data.get("token")
            if not token:
                raise ValidationError("'token' is required")

            ExpoToken.objects.update_or_create(user=user, token=token)

            # Remove oldest token if user exceeds token limit
            token_count = ExpoToken.objects.filter(user=user).count()
            if token_count > 10:
                oldest_token = ExpoToken.objects.filter(user=user).order_by("created_at").first()
                oldest_token.delete()

            return Response("token saved successfully")


        @action(detail=False, url_path="unregister-device", methods=["POST"])
        def unregister_device(self, request, *args, **kwargs):
            user = request.user

            token = request.data.get("token")
            if not token:
                raise ValidationError("token is required")

            ExpoToken.objects.filter(user=user, token=token).delete()

            return Response("token deleted successfully")


#!/usr/bin/env python

from setuptools import find_packages, setup

with open('README.md', 'r', encoding='utf-8') as f:
    description = f.read()

about = {}
with open('django_notifications_views/__version__.py', 'r', encoding="utf8") as f:
    exec(f.read(), about)

setup(
    name='django_notifications_views',
    version=about['__version__'],
    author='nachoborrelli',
    author_email='nacho2911@hotmail.com',
    url='https://github.com/nachoborrelli/Django-notifications-views',
    description='Django-notifications-views is an extension for Django-notifications-hq that provides a viewset for the notifications.',
    license='MIT',
    packages=find_packages(),
    long_description=description,
    long_description_content_type='text/markdown',
    keywords='django notifications hq views',
    zip_safe=False,
    install_requires=[
        'Django>=3.8.0',
        'pandas>=2.2.0',
        'requests>=2.30',
        'exponent-server-sdk~=2.0.0',
        'django-notifications-hq~=1.8.0',
    ],
    include_package_data=True,
    python_requires='>=3.8',
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ],
)



from django.conf import settings

USER_SETTINGS = getattr(settings, "DJANGO_NOTIFICATIONS_VIEWS", None)

DEFAULTS = {
    'USE_EXPO_NOTIFICATIONS': False,
    'EXPO_APP_ID': '', 
}

class APISettings:
    def __init__(self, user_settings=None, defaults=None):
        self.defaults = defaults
        self._user_settings = self.__check_user_settings(user_settings) \
            if user_settings else {}

    def __check_user_settings(self, user_settings):
        for setting in user_settings:
            if setting not in self.defaults:
                raise RuntimeError(f"The {setting} setting is not a valid setting for DJANGO_NOTIFICATIONS_VIEWS.")

        return user_settings

    def get_user_setting(self, attr):
        try:
            return self._user_settings[attr]
        except KeyError:
            return self.defaults[attr]


api_settings = APISettings(USER_SETTINGS, DEFAULTS)


